/*-
 * Copyright (c) 1990, 1991, 1992, 1993, 1994
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from the Stanford/CMU enet packet filter,
 * (net/enet.c) distributed as part of 4.3BSD, and code contributed
 * to Berkeley by Steven McCanne and Van Jacobson both of Lawrence
 * Berkeley Laboratory.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)bpf.c	7.5 (Berkeley) 7/15/91
 *
 * static char rcsid[] =
 * "$Header: bpf.c,v 1.56 94/06/28 20:23:47 mccanne Locked $";
 */

#ifdef BPFPROF

#include <machine/clock.h>

struct bprof_rec nit_rec[BPFPROF_MAXLEN];
struct bprof_rec bpf_rec[BPFPROF_MAXLEN];

bprof_reset()
{
	register int i, s = splimp();
	/* let clock free run */
	COUNTER->limit14 = 0;
	for (i = 0; i < BPFPROF_MAXLEN; ++i) {
		bpf_rec[i].br_min = ~0;
		bpf_rec[i].br_max = 0;
		bpf_rec[i].br_cnt = 0;
		bpf_rec[i].br_tv.tv_sec = 0;
		bpf_rec[i].br_tv.tv_usec = 0;

		nit_rec[i].br_min = ~0;
		nit_rec[i].br_max = 0;
		nit_rec[i].br_cnt = 0;
		nit_rec[i].br_tv.tv_sec = 0;
		nit_rec[i].br_tv.tv_usec = 0;
	}
	splx(s);
}

/*
 * Find the time difference from between two values read from
 * the level 14 counter register, which is is free running mode.
 * We assume the values are less than 1 second or so apart.
 */
static int
xtime(b, a)
	register int b, a;
{
	int ua = (a & CTR_USEC_MASK) >> CTR_USEC_SHIFT;
	int ub = (b & CTR_USEC_MASK) >> CTR_USEC_SHIFT;
	int delta = ua - ub;
	if (delta < 0)
		/*
		 * The after time wrapped around, but the before
		 * time hadn't.  Add in a whole period.
		 */
		delta += (CTR_USEC_MASK >> CTR_USEC_SHIFT) + 1;
	return (delta);
}

/*
 * Add u microseconds (u < 1000000) to the timeval pointed to by p.
 */
tvaddu(p, d)
	register struct timeval *p;
	register int d;
{
	p->tv_usec += d;
	if (p->tv_usec >= 1000000) {
		p->tv_usec -= 1000000;
		++p->tv_sec;
	}
}

prof_gen(b, a, len, br)
	register int b, a, len;
	struct bprof_rec *br;
{
	register int d = xtime(b, a);
	if (len >= BPFPROF_MAXLEN)
		len = BPFPROF_MAXLEN - 1;
	br += len;
	++br->br_cnt;
	if (d > br->br_max)
		br->br_max = d;
	if (d < br->br_min)
		br->br_min = d;
	tvaddu(&br->br_tv, xtime(b, a));
}

prof_bpf(b, a, len)
	register int b, a, len;
{
	prof_gen(b, a, len, bpf_rec);
}

prof_nit(b, a, len)
	register int b, a, len;
{
	prof_gen(b, a, len, nit_rec);
}

bprof_get(ip)
	struct bprof_info *ip;
{
	(void)copyout((void *)nit_rec, (void *)ip->bi_nit, sizeof(nit_rec));
	(void)copyout((void *)bpf_rec, (void *)ip->bi_bpf, sizeof(bpf_rec));
}
#endif
