/*
 * Copyright (c) 1996, 1997
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static const char copyright[] =
    "@(#) Copyright (c) 1996, 1997\n\
The Regents of the University of California.  All rights reserved.\n";
static const char rcsid[] =
    "@(#)$Header: setbpfilter.c,v 1.3 97/06/10 23:12:58 leres Exp $ (LBL)";
#endif

/*
 * pcapture - capture the last few packets
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>

#include <errno.h>
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include <pcap.h>
#include <signal.h>
#include <stdio.h>
#ifdef __STDC__
#include <stdlib.h>
#endif
#include <string.h>
#include <unistd.h>

#include "interface.h"
#include "machdep.h"

#include "pcap-int.h"

/* Globals */
char *program_name;

/* Locals */
static pcap_t *p;

/* Forwards */
static __dead void usage(void) __attribute__((volatile));

/* XXX */
void bpf_dump(struct bpf_program *, int);

extern int optind;
extern int opterr;
extern char *optarg;

int
main(int argc, char **argv)
{
	register int op, Oflag, snaplen, dflag, cmd;
	register char *cp, *infile, *expression, *device;
	u_int32_t localnet, netmask;
	struct bpf_program fcode;
	char ebuf[PCAP_ERRBUF_SIZE];

	device = NULL;
	infile = NULL;
	Oflag = 1;
	dflag = 0;
	cmd = BIOCSTCPF;		/* Default to tcp */
	snaplen = DEFAULT_SNAPLEN;
	if ((cp = strrchr(argv[0], '/')) != NULL)
		program_name = cp + 1;
	else
		program_name = argv[0];

	if (abort_on_misalignment(ebuf) < 0)
		error(ebuf);

	opterr = 0;
	while ((op = getopt(argc, argv, "dOpTUF:i:s:")) != EOF)
		switch (op) {

		case 'd':
			++dflag;
			break;

		case 'F':
			infile = optarg;
			break;

		case 'i':
			/* Undocumented */
			device = optarg;
			break;

		case 'O':
			Oflag = 0;
			break;

		case 's':
			/* Undocumented */
			snaplen = atoi(optarg);
			break;

		case 'T':
			cmd = BIOCSTCPF;
			break;

		case 'U':
			cmd = BIOCSUDPF;
			break;

		default:
			usage();
			/* NOTREACHED */
		}

	/* Determine the device name if not specified */
	if (device == NULL && (device = pcap_lookupdev(ebuf)) == NULL)
		error(ebuf);

	/* Fire up libpcap */
	p = pcap_open_live(device, snaplen, 0, 1000, ebuf);
	if (p == NULL)
		error(ebuf);
	if (pcap_lookupnet(device, &localnet, &netmask, ebuf) < 0)
		error(ebuf);

	/* Construct, compile and set filter */
p->linktype = DLT_RAW;
	if (infile)
		expression = read_infile(infile);
	else
		expression = copy_argv(argv + optind);
	if (expression == NULL || *expression == '\0')
		memset(&fcode, 0, sizeof(fcode));
	else if (pcap_compile(p, &fcode, expression, Oflag, netmask) < 0 ||
	    pcap_setfilter(p, &fcode) < 0)
		error(pcap_geterr(p));
	
	if (dflag) {
		bpf_dump(&fcode, dflag);
		exit(0);
	}

#ifdef notdef
	(void)fprintf(stderr, "%s: listening on %s\n", program_name, device);
	(void)fflush(stderr);
#endif

	if (ioctl(p->fd, cmd, (char *)&fcode) < 0)
		error("%s: %s", cmd == BIOCSTCPF ? "BIOCSTCPF" : "BIOCSUDPF",
		    pcap_strerror(errno));

	exit(0);
}

static __dead void
usage()
{

	(void)fprintf(stderr, "Version %s\n", version);
	(void)fprintf(stderr,
"Usage: %s [-Op] [-c count] [ -F file ] [ -i interface ]\n", program_name);
	(void)fprintf(stderr,
"\t\t[ -s snaplen ] -w file [ expression ]\n");
	exit(-1);
}
